@@ -2,7 +2,6 @@
 Classes for storing configuration by namespace, and reloading configuration
 while files change.
 """
-import functools
 import logging
 import os
 import time
@@ -120,7 +119,6 @@ class ConfigNamespace(object):
         return "%s(%s)" % (type(self).__name__, self.name)
 
 
-config_key_descriptions = {}
 configuration_namespaces = {DEFAULT: ConfigNamespace(DEFAULT)}
 
 
@@ -151,11 +149,6 @@ def reload(name=DEFAULT, all_names=False):
             value_proxy.reset()
 
 
-def add_config_key_description(name, validator, default, namespace, help):
-    desc = KeyDescription(name, validator, default, help)
-    config_key_descriptions.setdefault(namespace, []).append(desc)
-
-
 def validate(name=DEFAULT, all_names=False):
     """Access values in all registered proxies in a namespace. Missing values
     raise ConfigurationError. Defaults to the DEFAULT namespace. If all_names
@@ -165,36 +158,54 @@ def validate(name=DEFAULT, all_names=False):
         all(bool(value_proxy) for value_proxy in namespace.get_value_proxies())
 
 
-def view_help():
-    """Return a help message describing all the statically configured keys.
-    """
-    def format(desc):
-        help        = desc.help or ''
-        type_name   = desc.validator.__name__.replace('validate_', '')
-        fmt         = "%s (Type: %s, Default: %s)\n%s"
-        return fmt % (desc.name, type_name, desc.default, help)
+class ConfigHelp(object):
+    """Register and display help messages about config keys."""
 
-    def format_namespace(key, desc_list):
-        fmt = "\nNamespace: %s\n%s"
-        seq = sorted(format(desc) for  desc in desc_list)
-        return fmt % (key, '\n'.join(seq))
+    def __init__(self):
+        self.descriptions = {}
 
-    def namespace_sort(lhs, rhs):
-        if lhs == DEFAULT:
-            return -1
-        if rhs == DEFAULT:
-            return 1
-        return lhs < rhs
+    def add(self, name, validator, default, namespace, help):
+        desc = KeyDescription(name, validator, default, help)
+        self.descriptions.setdefault(namespace, []).append(desc)
 
-    seq = sorted(config_key_descriptions.iteritems(), cmp=namespace_sort)
-    return '\n'.join(format_namespace(*desc) for desc in seq)
+    def view_help(self):
+        """Return a help message describing all the statically configured keys.
+        """
+        def format(desc):
+            help        = desc.help or ''
+            type_name   = desc.validator.__name__.replace('validate_', '')
+            fmt         = "%s (Type: %s, Default: %s)\n%s"
+            return fmt % (desc.name, type_name, desc.default, help)
+
+        def format_namespace(key, desc_list):
+            fmt = "\nNamespace: %s\n%s"
+            seq = sorted(format(desc) for  desc in desc_list)
+            return fmt % (key, '\n'.join(seq))
+
+        def namespace_sort(lhs, rhs):
+            if lhs == DEFAULT:
+                return -1
+            if rhs == DEFAULT:
+                return 1
+            return lhs < rhs
+
+        seq = sorted(self.descriptions.iteritems(), cmp=namespace_sort)
+        return '\n'.join(format_namespace(*desc) for desc in seq)
+
+    def clear(self):
+        self.descriptions.clear()
+
+
+config_help = ConfigHelp()
+view_help = config_help.view_help
 
 
 def _reset():
     """Used for internal testing."""
     for namespace in configuration_namespaces.values():
         namespace._reset()
-    config_key_descriptions.clear()
+    config_help.clear()
+
 
 
 def has_duplicate_keys(config_data, base_conf, raise_error):
@@ -237,11 +248,9 @@ class ConfigurationWatcher(object):
         return max(os.path.getmtime(name) for name in self.filenames)
 
     def _get_inodes(self):
-        values = []
-        for filename in self.filenames:
-            stbuf = os.stat(filename)
-            values.append((stbuf.st_dev, stbuf.st_ino))
-        return values
+        def get_inode(stbuf):
+            return stbuf.st_dev, stbuf.st_ino
+        return [get_inode(os.stat(filename)) for filename in self.filenames]
 
     def reload_if_changed(self, force=False):
         if (force or self.should_check) and self.file_modified():
@@ -261,6 +270,9 @@ class ConfigurationWatcher(object):
     def get_reloader(self):
         return self.reloader
 
+    def load_config(self):
+        return self.config_loader()
+
 
 class ReloadCallbackChain(object):
     """This object can be used as a convenient way of adding and removing
@@ -284,6 +296,13 @@ class ReloadCallbackChain(object):
             callback()
 
 
+def build_loader_callable(load_func, filename, namespace):
+    def load_configuration():
+        get_namespace(namespace).clear()
+        return load_func(filename, namespace=namespace)
+    return load_configuration
+
+
 class ConfigFacade(object):
     """A facade around a ConfigurationWatcher and a ReloadCallbackChain.
     """
@@ -294,11 +313,12 @@ class ConfigFacade(object):
 
     @classmethod
     def load(cls, filename, namespace, loader_func, min_interval=0):
-        loader = functools.partial(loader_func, filename, namespace=namespace)
-        reloader = ReloadCallbackChain(namespace=namespace)
         watcher = ConfigurationWatcher(
-            loader, filename, min_interval=min_interval, reloader=reloader)
-        loader()
+            build_loader_callable(loader_func, filename, namespace=namespace),
+            filename,
+            min_interval=min_interval,
+            reloader=ReloadCallbackChain(namespace=namespace))
+        watcher.load_config()
         return cls(watcher)
 
     def add_callback(self, identifier, callback):
